
local AIs = { --Angle limitations of each AI
{0,1.57},
{1.57,3.14},
{3.14,4.71},
}

function updateAIs(dt)
	for i=1,#AIs do
		local target = closestAsteroid(i)
		if (target and AIs[i].planet.alive) then --Check they can fire
			--Find targets
			local targetPositions = {}
			AIs[i].target.time = AIs[i].target.time-dt
			if AIs[i].target.time<=0 then --New target if the old one expired
				for j=1,#planets do
					if planets[j] ~= AIs[i].planet then
						--Values for equations of circle + line
						local a,b = AIs[i].satellite.center.x,AIs[i].satellite.center.y
						local m = (planets[j].position.y-target.position.y)/(planets[j].position.x-target.position.x)
						local c = planets[j].position.y/(m*planets[j].position.x)
						local r = AIs[i].satellite.distance
						
						--Quadratic equation
						local A,B,C = (m^2 +1), 2*(m*c-m*b-a), b^2 - r^2 + a^2 - 2*c*b + c^2
						
						--Calculate point(s) of intersection
						if B^2 - 4*A*C==0 then
							local targx = ((-B+math.sqrt(B^2 - 4*A*C))/2*A)
							local targy = m*targx + c
							local angle = math.atan2(targy,targx)
							if angle>=AIs[i][1] and angle<=AIs[i][2] then
								table.insert(targetPositions,{target,angle})
								AIs[i].target.time = math.random()
							end
						elseif B^2 - 4*A*C>0 then
							--Find the point that is usable (if any)
							local targx1 = ((-B+math.sqrt(B^2 - 4*A*C))/2*A)
							local targx2 = ((-B-math.sqrt(B^2 - 4*A*C))/2*A)
							local targy1 = m*targx1 + c
							local targy2 = m*targx2 + c
							local angle1 = math.atan2(targy1,targx1)
							local angle2 = math.atan2(targy2,targx2)
							if angle1>=AIs[i][1] and angle1<=AIs[i][2] then
								table.insert(targetPositions,{target,angle1})
								AIs[i].target.time = math.random()
							elseif angle2>=AIs[i][1] and angle2<=AIs[i][2] then
								table.insert(targetPositions,{target,angle2})
								AIs[i].target.time = math.random()
							end
						else --Just fire randomly
							local angle = math.random(AIs[i][1],AIs[i][2])
							table.insert(targetPositions,{target,angle})
							AIs[i].target.time = math.random()
						end
					end
				end
				if #targetPositions>0 then
					--Select a new target
					local targ = targetPositions[math.random(1,#targetPositions)]
					AIs[i].target.asteroid = targ[1]
					AIs[i].target.angle = targ[2]
				end
			end
			--Move into position
			if AIs[i].target.angle>AIs[i].satellite.angle then
				AIs[i].satellite.angle = (AIs[i].satellite.angle+1.57*dt)%6.28
			elseif AIs[i].target.angle<AIs[i].satellite.angle then
				AIs[i].satellite.angle = (AIs[i].satellite.angle-1.57*dt)%6.28
			end
			if AIs[i].satellite.angle<AIs[i][1] then
				AIs[i].satellite.angle = AIs[i][1]
			end
			if AIs[i][1]==0 then
				if AIs[i].satellite.angle>5 then --Special case
					AIs[i].satellite.angle = 0
				end
			end
			if AIs[i].satellite.angle>AIs[i][2] then
				AIs[i].satellite.angle = AIs[i][2]
			end
			AIs[i].satellite.position = angleToPosition(AIs[i].satellite.angle)*AIs[i].satellite.distance + AIs[i].satellite.center
			if AIs[i].target.asteroid then
				fireBeam(AIs[i].satellite,AIs[i].target.asteroid.position,dt)
			else
				AIs[i].satellite.beamEnd = nil
				AIs[i].satellite.target = nil
			end
		else
			AIs[i].satellite.beamEnd = nil
			AIs[i].satellite.target = nil
		end
	end
end

function drawAIs()
	for i=1,#AIs do
		local satellite = AIs[i].satellite
		love.graphics.draw(sat,satellite.position.x,satellite.position.y,0,0.1,0.1,sat:getWidth()/2,sat:getHeight()/2)
		if satellite.target then
			love.graphics.setColor(255,255,255,50)
			love.graphics.setLineWidth(3)
			love.graphics.line(satellite.position.x,satellite.position.y,satellite.target.position.x,satellite.target.position.y)
			for i=1,5 do
				love.graphics.circle("fill",satellite.target.position.x,satellite.target.position.y,i*((satellite.target.radius+10)/5))
			end
		elseif satellite.beamEnd then
			love.graphics.setColor(255,255,255,50)
			love.graphics.setLineWidth(3)
			love.graphics.line(satellite.position.x,satellite.position.y,satellite.beamEnd.x,satellite.beamEnd.y)
		end
		love.graphics.setColor(255,255,255)
	end
end

function resetAIs()
	for i=1,#AIs do
		AIs[i].satellite = {}
		AIs[i].satellite.distance = 180
		local p = i==3 and 4 or i --Third planet is the players, so we skip it
		AIs[i].satellite.center = planets[p].position
		AIs[i].satellite.angle = (AIs[i][1]+AIs[i][2])/2
		AIs[i].satellite.position = angleToPosition(AIs[i].satellite.angle)*AIs[i].satellite.distance + AIs[i].satellite.center
		AIs[i].planet = planets[p]
		AIs[i].target = {}
		AIs[i].target.time = 0
		AIs[i].target.angle = AIs[i].satellite.angle
	end
end

function closestAsteroid(AIID)
	local closest = nil
	local dist = 900/difficulty --Don't bother with those out of range
	for i=1,#asteroids do
		if (asteroids[i].position-AIs[AIID].satellite.position):magnitude()<dist then
			closest = asteroids[i]
			dist = (asteroids[i].position-AIs[AIID].satellite.position):magnitude()
		end
	end
	return closest
end