local res = {}
res.desiredRes = {800,600}

res.scale = love.graphics.scale

res.getMouse = love.mouse.getPosition
res.getX = love.mouse.getX
res.getY = love.mouse.getY

res.setMouse = love.mouse.setPosition

res.getWidth = love.graphics.getWidth
res.getHeight = love.graphics.getHeight

res.setScissor = love.graphics.setScissor

res.setMode = love.graphics.setMode

function res.draw(self)
	love.graphics.push()
	if self.letterBox==1 then
		--Stop user drawing outside of "window"
		self.setScissor(self.letterSize,0,self.currentRes[1]-self.letterSize*2,self.currentRes[2])
		love.graphics.translate(self.letterSize,0)
	else
		--Stop user drawing outside of "window"
		self.setScissor(0,self.letterSize,self.currentRes[1],self.currentRes[2]-self.letterSize*2)
		love.graphics.translate(0,self.letterSize)
	end
	self.scale(self.scaleRate,self.scaleRate)
end

function res.undraw(self)
	love.graphics.pop()
	self.setScissor()
	love.graphics.setColor(0,0,0)
	if self.letterBox==1 then
		--Draw letterbox
		love.graphics.rectangle("fill",0,0,self.letterSize,self.currentRes[2])
		love.graphics.rectangle("fill",self.currentRes[1]-self.letterSize,0,self.letterSize,self.currentRes[2])
	else
		--Draw letterbox
		love.graphics.rectangle("fill",0,0,self.currentRes[1],self.letterSize)
		love.graphics.rectangle("fill",0,self.currentRes[2]-self.letterSize,self.currentRes[1],self.letterSize)
	end
	love.graphics.setColor(255,255,255)
end

function res.update(self)
	self.currentRes = {self.getWidth(),self.getHeight()}
	local scaleX = self.currentRes[1]/self.desiredRes[1]
	local scaleY = self.currentRes[2]/self.desiredRes[2]
	
	if scaleY*self.desiredRes[1]<self.currentRes[1] then
		res.letterBox = 1 --X axis needs boxing
		res.letterSize = (self.currentRes[1]-scaleY*self.desiredRes[1])/2 --Halved so it fits each side
		res.scaleRate = scaleY
	else
		res.letterBox = 2 --Y axis needs boxing
		res.letterSize = (self.currentRes[2]-scaleX*self.desiredRes[2])/2 --Halved so it fits each side
		res.scaleRate = scaleX
	end
end

function res.setResolution(self,x,y)
	self.desiredRes = {x,y}
	res:update()
end

res:update()

function love.mouse.getPosition()
	local x,y = res.getMouse()
	if res.letterBox == 1 then
		x = x - res.letterSize
	else
		y = y - res.letterSize
	end
	x = x/res.scaleRate
	y = y/res.scaleRate
	return x,y
end

function love.mouse.getX()
	local x = res.getX()
	if res.letterBox == 1 then
		x = x - res.letterSize
	end
	x = x/res.scaleRate
	return x
end

function love.mouse.getY()
	local y = res.getY()
	if res.letterBox == 2 then
		y = y - res.letterSize
	end
	y = y/res.scaleRate
	return y
end

function love.mouse.setPosition(x,y)
	if res.letterBox == 1 then
		x = x - res.letterSize
	else
		y = y - res.letterSize
	end
	x = x/res.scaleRate
	y = y/res.scaleRate
	res.setMouse(x,y)
end

function love.graphics.scale(x,y)
	if not y then y = x end
	res.scale(x*res.scaleRate,y*res.scaleRate) --Prevent scaling from messing up resolution
end

function love.graphics.getWidth()
	return res.desiredRes[1]
end

function love.graphics.getHeight()
	return res.desiredRes[2]
end

function love.graphics.setScissor(x,y)
	if not x then --Reset scissor
		if res.letterBox==1 then
			res.setScissor(res.letterSize,0,res.currentRes[1]-res.letterSize*2,res.currentRes[2]) 
		else
			res.setScissor(0,res.letterSize,res.currentRes[1],res.currentRes[2]-res.letterSize*2)
		end
	else
		if res.letterBox == 1 then
			x = x - res.letterSize
		else
			y = y - res.letterSize
		end
		x = x*res.scaleRate
		y = y*res.scaleRate
		res.setScissor(x,y)
	end
end

function love.graphics.setMode(...)
	local args = {...}
	if args[1] == 0 or args[2] == 0 then
		res.setMode(0,0)
		args[1] = res.getWidth()
		args[2] = res.getHeight()
	end
	res.setMode(unpack(args))
	res:update()
end
return res