
local vec = {}
local meta = {}

function vec.new(self,x,y)
	local o = {}
	meta.__index = vec
	setmetatable(o, meta)
	o.x = x
	o.y = y
	self.__index = self
	return o
end

--Returns the sum of two vectors
--Returns an offset vector in the x and y axis by the number

function meta.__add(a,b)
	if type(a) == "number" and type(b) == "table" then
		local a2,b2 = a,b --Swap them the right way round
		a = b2
		b = a2
	end
	if type(b)=="table" and b.x then
		return vec:new(a.x+b.x,a.y+b.y)
	elseif tonumber(b) then
		return vec:new(a.x+b,a.y+b)
	else
		error("Attempted to add "..tostring(b).." to a vector",2)
	end
end

--Returns the difference of two vectors
--Returns an offset vector in the x and y axis by the number

function meta.__sub(a,b)
	if type(a) == "number" and type(b) == "table" then
		local a2,b2 = a,b --Swap them the right way round
		a = b2
		b = a2
	end
	if type(b)=="table" and b.x then
		return vec:new(a.x-b.x,a.y-b.y)
	elseif tonumber(b) then
		return vec:new(a.x-b,a.y-b)
	else
		error("Attempted to subtract "..tostring(b).." from a vector",2)
	end
end

--By multiplying two vectors you get the dot product
--By using a vector then a number you get a scaled vector

function meta.__mul(a,b)
	if type(a) == "number" and type(b) == "table" then
		local a2,b2 = a,b --Swap them the right way round
		a = b2
		b = a2
	end
	if type(b)=="table" and b.x then
		return a.x * b.x + a.y * b.y
	elseif tonumber(b) then
		return vec:new(a.x*b,a.y*b)
	else
		error("Attempted to multiply "..tostring(b).." with a vector",2)	
	end
end

--Returns a scaled down vector by the number provided

function meta.__div(a,b)
	if tonumber(b) then
		return vec:new(a.x/b,a.y/b)
	else
		error("Attempted to divide "..tostring(b).." with a vector",2)	
	end
end

--Returns if the vectors are the same

function meta.__eq(a,b)
	return a.x==b.x and a.y==b.y
end

--Returns the length of the vector

function vec.magnitude(a)
	return math.sqrt(a.x^2+a.y^2)
end

--Returns the normalised size of the vector

function vec.unit(a)
	return a/a:magnitude()
end

--Returns a human readable interpretation of the vector

function meta.__tostring(a)
	return "Vector ("..a.x..", "..a.y..")"
end

return vec