
vector = require "lib.vector"
resolution = require "lib.resolution"

require "asteroid"
require "planet"
require "satellite"
require "ai"

difficulty = 2

local background = love.graphics.newImage("assets/spaceBack.png")
local aster = love.graphics.newImage("assets/asteroid.png")

local startTime = 3

local diffSelect = true

function love.load(t)
	if (not t) or t~="restart" then
		love.graphics.setFont(love.graphics.newFont(72))
		resolution.desiredRes = {800,800}
		love.graphics.setMode(600,600)
		love.graphics.setIcon(aster)
	end
	generateAsteroids()
	generatePlanets()
	resetSatellite()
	resetAIs()
	startTime = 3
	diffSelect = true
end

function love.draw()
	resolution:draw()
	love.graphics.draw(background)
	if diffSelect then
		love.graphics.print("Difficulty:"..difficulty,200,300)
		love.graphics.print("Press enter to start, ESC to quit",280,400,0,0.2,0.2)
		resolution:undraw()
		return
	end
	drawPlanets()
	drawAsteroids()
	drawSatellite()
	drawAIs()
	if startTime>0 then
		love.graphics.print(math.ceil(startTime),400,400,0,1/(math.ceil(startTime)-startTime),1/(math.ceil(startTime)-startTime),20,40)
	end
	if not planets[3].alive then
		love.graphics.print("Game Over",400,300,0,1,1,200,0)
		love.graphics.print("Press R to restart, ESC to quit",300,400,0,0.2,0.2)
	end
	if not (planets[1].alive or planets[2].alive or planets[4].alive) then
		love.graphics.print("All planets destroyed",20,300,0,1,1)
		love.graphics.print("Press R to restart, ESC to quit",300,400,0,0.2,0.2)
	end
	resolution:undraw()
end

function love.update(dt)
	dt = math.min(dt,1/60)
	if diffSelect then
		return
	end
	if startTime>0 then
		startTime = startTime-dt
	elseif planets[3].alive and (planets[1].alive or planets[2].alive or planets[4].alive) then --If someone is alive and so is the player
		updateAsteroids(dt)
		updateSatellite(dt)
		updateAIs(dt)
	end
end

function love.keypressed(k,u)
	if k=="=" then
		love.graphics.setMode(resolution.getWidth()+20,resolution.getWidth()+20)
	end
	if k=="-" then
		love.graphics.setMode(resolution.getWidth()-20,resolution.getWidth()-20)
	end
	if k=="escape" then
		love.event.quit()
	end
	if k=="r" and not planets[3].alive then --Restart if game over
		love.load("restart")
	end
	if diffSelect and k=="a" or k=="left" then
		difficulty = (difficulty-2)%3 +1
	end
	if diffSelect and k=="d" or k=="right" then
		difficulty = difficulty%3 +1
	end
	if diffSelect and k=="return" then
		diffSelect = false
	end
end
