local satellite = {
}
sat = love.graphics.newImage("assets/satellite.png")

function resetSatellite()
satellite = {}
satellite.distance = 180 --distance from the planet
satellite.center = vector:new(0,800)
satellite.angle = 5.5
satellite.position = angleToPosition(satellite.angle)*satellite.distance + satellite.center --Move it from the centre of its planet
end

function updateSatellite(dt)
	if love.keyboard.isDown("left","a") then
		satellite.angle = math.max(4.71,(satellite.angle-1.57*dt)%6.28)
	end
	if love.keyboard.isDown("right","d") then
		satellite.angle = (satellite.angle+1.57*dt)%6.28
		if satellite.angle<4.71 then satellite.angle = 6.28 end
	end
	
	satellite.position = angleToPosition(satellite.angle)*satellite.distance + satellite.center
	
	if love.mouse.isDown("l") then --Beam firing
		fireBeam(satellite,vector:new(love.mouse.getX(),love.mouse.getY()),dt)
	else
		satellite.target = nil
		satellite.beamEnd = nil
	end
end

function drawSatellite()
	love.graphics.draw(sat,satellite.position.x,satellite.position.y,0,0.1,0.1,sat:getWidth()/2,sat:getHeight()/2)
	if satellite.target then
		love.graphics.setColor(255,255,255,50)
		love.graphics.setLineWidth(3)
		love.graphics.line(satellite.position.x,satellite.position.y,satellite.target.position.x,satellite.target.position.y)
		for i=1,5 do
			love.graphics.circle("fill",satellite.target.position.x,satellite.target.position.y,i*((satellite.target.radius+10)/5))
		end
	elseif satellite.beamEnd then
		love.graphics.setColor(255,255,255,50)
		love.graphics.setLineWidth(3)
		love.graphics.line(satellite.position.x,satellite.position.y,satellite.beamEnd.x,satellite.beamEnd.y)
	end
	love.graphics.setColor(255,255,255)
end

--Used to make the satellites travel in an arc
function angleToPosition(ang)
	return vector:new(
	math.cos(ang),
	math.sin(ang)
	)
end

function fireBeam(sat,pos,dt)
	sat.beamEnd = pos
	local length = 900/difficulty --Length the beam can fire, shortens with difficulty
	local closest = nil
	local distance = nil
	for i=1,#asteroids do
		if beamHits(sat.position,sat.beamEnd,length,asteroids[i]) then
			if not distance or (asteroids[i].position-sat.position):magnitude()<distance then
				distance = (asteroids[i].position-sat.position):magnitude()
				closest = asteroids[i]
			end
		end
	end
	if closest then
		closest.velocity = closest.velocity + (closest.position-sat.position):unit()*dt*500--(closest.position-satellite.position)*dt
		sat.target = closest
	else
		sat.target = nil
	end
end

--Calculates if the beam hit the asteroid
function beamHits(beamStart,beamEnd,length,asteroid)
	local grad1 = (beamEnd.y-beamStart.y)/(beamEnd.x-beamStart.x)
	local offset1 = beamEnd.y-(grad1*beamEnd.x)
	
	local grad2 = -1/grad1
	local offset2 = asteroid.position.y-(grad2*asteroid.position.x)
	
	local x = (offset1-offset2)/(grad2-grad1)
	local coll = vector:new(x,grad1*x+offset1)
	coll = grad1==0 and vector:new(asteroid.position.x,beamEnd.y) or coll
	coll = grad2==0 and vector:new(beamEnd.x,asteroid.position.y) or coll
	
	local dist = (coll-asteroid.position):magnitude()
	local distfromsat = (beamStart-coll):magnitude()
	local unita = (coll-beamStart):unit()
	local unitb = (beamEnd-beamStart):unit()
	unita.x = math.floor(unita.x*100)/100
	unita.y = math.floor(unita.y*100)/100
	unitb.x = math.floor(unitb.x*100)/100
	unitb.y = math.floor(unitb.y*100)/100

	return dist<=asteroid.radius and distfromsat<=length and unita == unitb
end
